#include <stdlib.h>
#include "ChaosAbort.h"
#include "ChaosAndroid.h"
#include "NemesisJavaScript.h"
#include "ThanatosAndroid.h"
#include "AndroidPluginCommon.h"
#include "AdjustPlugin.h"

void AdjustTrackEvent( void* args )
{
	( void )args;
	
	JNIEnv* env = ChaosAndroidGetEnvironment();
	char const* token = NemesisJavaScriptPopString( NemesisJavaScriptPhaseUpdate );
	
	jstring jToken = ( *env )->NewStringUTF( env, token );
	if ( 0 == jToken )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	jclass adjustEvent = CreateJClass( "com.adjust.sdk.AdjustEvent" );
	jmethodID init = ( *env )->GetMethodID( env, adjustEvent, "<init>", "(Ljava/lang/String;)V" );
	if ( init == 0 )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	
	jobject adjustEventObject = ( *env )->NewObject( env, adjustEvent, init, jToken );
	if ( 0 == adjustEventObject )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	( *env )->DeleteLocalRef( env, jToken );
	
	jmethodID addCallbackParameter = ( *env )->GetMethodID( env, adjustEvent, "addCallbackParameter", "(Ljava/lang/String;Ljava/lang/String;)V" );
	if( 0 == addCallbackParameter )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	DestroyJClass( adjustEvent );
	
	uint32_t size = NemesisJavaScriptPopInteger( NemesisJavaScriptPhaseUpdate );
	uint32_t index;
	for( index = 0; index != size; index++ )
	{
		char const* key = NemesisJavaScriptPopString( NemesisJavaScriptPhaseUpdate );
		char const* value = NemesisJavaScriptPopString( NemesisJavaScriptPhaseUpdate );
		jstring jKey = ( *env )->NewStringUTF( env, key );
		if ( 0 == jKey )
		{
			ChaosAbort( __FILE__, __LINE__ );
		}
		jstring jValue = ( *env )->NewStringUTF( env, value );
		if ( 0 == jValue )
		{
			ChaosAbort( __FILE__, __LINE__ );
		}
		( *env )->CallVoidMethod( env, adjustEventObject, addCallbackParameter, jKey, jValue );
		( *env )->DeleteLocalRef( env, jValue );
		( *env )->DeleteLocalRef( env, jKey );
	}
	
	jclass adjust = CreateJClass( "com.adjust.sdk.Adjust" );
	jmethodID trackEvent = ( *env )->GetStaticMethodID( env, adjust, "trackEvent", "(Lcom/adjust/sdk/AdjustEvent;)V" );
	if ( 0 == trackEvent )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	( *env )->CallStaticVoidMethod( env, adjust, trackEvent, adjustEventObject );
	DestroyJClass( adjust );
	( *env )->DeleteLocalRef( env, adjustEventObject );
}

void AdjustTrackRevenue( void* args )
{
	( void )args;
	
	JNIEnv* env = ChaosAndroidGetEnvironment();
	char const* token = NemesisJavaScriptPopString( NemesisJavaScriptPhaseUpdate );
	char const* price = NemesisJavaScriptPopString( NemesisJavaScriptPhaseUpdate );
	char const* currency = NemesisJavaScriptPopString( NemesisJavaScriptPhaseUpdate );
	
	jstring jToken = ( *env )->NewStringUTF( env, token );
	if ( 0 == jToken )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	
	jdouble priceNumber = atof( price );
	jstring jCurrency = ( *env )->NewStringUTF(env, currency );
	if ( 0 == jCurrency )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	
	jclass adjustEvent = CreateJClass( "com.adjust.sdk.AdjustEvent" );
	jmethodID init = ( *env )->GetMethodID( env, adjustEvent, "<init>", "(Ljava/lang/String;)V" );
	if ( 0 == init )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	jobject adjustEventObject = ( *env )->NewObject( env, adjustEvent, init, jToken );
	if ( 0 == adjustEventObject )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	( *env )->DeleteLocalRef( env, jToken );
	jmethodID setRevenue = ( *env )->GetMethodID( env, adjustEvent, "setRevenue", "(DLjava/lang/String;)V" );
	if ( 0 == setRevenue )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	( *env )->CallVoidMethod( env, adjustEventObject, setRevenue, priceNumber, jCurrency );
	( *env )->DeleteLocalRef( env, jCurrency );
	
	jmethodID addCallbackParameter = ( *env )->GetMethodID( env, adjustEvent, "addCallbackParameter", "(Ljava/lang/String;Ljava/lang/String;)V" );
	if ( 0 == addCallbackParameter )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	DestroyJClass( adjustEvent );
	uint32_t size = NemesisJavaScriptPopInteger( NemesisJavaScriptPhaseUpdate );
	uint32_t index;
	for ( index = 0; index != size; index++ )
	{
		char const* key = NemesisJavaScriptPopString( NemesisJavaScriptPhaseUpdate );
		char const* value = NemesisJavaScriptPopString( NemesisJavaScriptPhaseUpdate );
		jstring jKey = ( *env )->NewStringUTF( env, key );
		if ( 0 == jKey )
		{
			ChaosAbort( __FILE__, __LINE__ );
		}
		jstring jValue = ( *env )->NewStringUTF( env, value );
		if ( 0 == jValue )
		{
			ChaosAbort( __FILE__, __LINE__ );
		}
		( *env )->CallVoidMethod( env, adjustEventObject, addCallbackParameter, jKey, jValue );
		( *env )->DeleteLocalRef( env, jValue );
		( *env )->DeleteLocalRef( env, jKey );
	}
	jclass adjust = CreateJClass( "com.adjust.sdk.Adjust" );
	jmethodID trackEvent = ( *env )->GetStaticMethodID( env, adjust, "trackEvent", "(Lcom/adjust/sdk/AdjustEvent;)V" );
	if ( 0 == trackEvent )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	( *env )->CallStaticVoidMethod( env, adjust, trackEvent, adjustEventObject );
	DestroyJClass( adjust );
	( *env )->DeleteLocalRef( env, adjustEventObject );
}

void AdjustInitialize()
{
	JNIEnv* env = ChaosAndroidGetEnvironment();
	jstring jToken = ( *env )->NewStringUTF( env, "floh6g1ll3b4" );
	
	if( 0 == jToken )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	jstring jEnvironment = ( *env )->NewStringUTF( env, "production" );
	if( 0 == jEnvironment )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}

	jclass typeAdjustConfiguration = CreateJClass( "com.adjust.sdk.AdjustConfig" );
	if( 0 == typeAdjustConfiguration )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	jmethodID methodId = ( *env )->GetMethodID( env, typeAdjustConfiguration, "<init>", "(Landroid/content/Context;Ljava/lang/String;Ljava/lang/String;)V" );
	if( 0 == methodId )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	jobject adjustConfiguration = ( *env )->NewObject( env, typeAdjustConfiguration, methodId, ThanatosAndroidGetActivityObject(), jToken, jEnvironment );
	if( 0 == adjustConfiguration )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	( *env )->DeleteLocalRef( env, jEnvironment );
	( *env )->DeleteLocalRef( env, jToken );
	
	jclass typeLogLevel = CreateJClass( "com.adjust.sdk.LogLevel" );
	jfieldID fieldId = ( *env )->GetStaticFieldID( env, typeLogLevel, "VERBOSE", "Lcom/adjust/sdk/LogLevel;" );
	if( 0 == fieldId )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	jobject logLevel = ( *env )->GetStaticObjectField( env, typeLogLevel, fieldId );
	if( 0 == logLevel )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	DestroyJClass( typeLogLevel );
	
	methodId = ( *env )->GetMethodID( env, typeAdjustConfiguration, "setLogLevel", "(Lcom/adjust/sdk/LogLevel;)V" );
	if( 0 == methodId )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	DestroyJClass( typeAdjustConfiguration );
	( *env )->CallVoidMethod( env, adjustConfiguration, methodId, logLevel );
	( *env )->DeleteLocalRef( env, logLevel );
	jclass adjust = CreateJClass( "com.adjust.sdk.Adjust" );
	methodId = ( *env )->GetStaticMethodID( env, adjust, "onCreate", "(Lcom/adjust/sdk/AdjustConfig;)V" );
	if( 0 == methodId )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	( *env )->CallStaticVoidMethod( env, adjust, methodId, adjustConfiguration );
	/*methodId = ( *env )->GetStaticMethodID( env, adjust, "onResume", "()V" );
	if( 0 == methodId )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	( *env )->CallStaticVoidMethod( env, adjust, methodId );
	methodId = ( *env )->GetStaticMethodID( env, adjust, "onPause", "()V" );
	if( 0 == methodId )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	( *env )->CallStaticVoidMethod( env, adjust, methodId );*/
	DestroyJClass( adjust );
	( *env )->DeleteLocalRef( env, adjustConfiguration );

	NemesisJavaScriptAppendCommand( NemesisJavaScriptPhaseUpdate, "AdjustTrackEvent", AdjustTrackEvent, 0 );
	NemesisJavaScriptAppendScript( "function AdjustTrackEvent(token,param){Nemesis.PushCommand(Nemesis.UpdatePhase,\"AdjustTrackEvent\");Nemesis.PushString(token);Nemesis.PushInteger(Object.keys(param).length);for(var key in param){Nemesis.PushString(key);Nemesis.PushString(param[key]);}};" );

	NemesisJavaScriptAppendCommand( NemesisJavaScriptPhaseUpdate, "AdjustTrackRevenue", AdjustTrackRevenue, 0 );
	NemesisJavaScriptAppendScript( "function AdjustTrackRevenue( token, price, currency, param ){Nemesis.PushCommand(Nemesis.UpdatePhase,\"AdjustTrackRevenue\");Nemesis.PushString(token);Nemesis.PushString(price);Nemesis.PushString(currency);Nemesis.PushInteger(Object.keys(param).length);for(var key in param){Nemesis.PushString(key);Nemesis.PushString(param[key]);}};" );
}

void AdjustFinalize()
{
}

void AdjustStartCallback()
{
	JNIEnv* env = ChaosAndroidGetEnvironment();
	jclass adjust = CreateJClass( "com.adjust.sdk.Adjust" );
	
	jmethodID onResume = ( *env )->GetStaticMethodID( env, adjust, "onResume", "()V" );
	if ( 0 == onResume )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	
	( *env )->CallStaticVoidMethod( env, adjust, onResume );
	DestroyJClass( adjust );
}

void AdjustStopCallback()
{
	JNIEnv* env = ChaosAndroidGetEnvironment();
	jclass adjust = CreateJClass( "com.adjust.sdk.Adjust" );
	
	jmethodID onPause = ( *env )->GetStaticMethodID( env, adjust, "onPause", "()V" );
	if ( 0 == onPause )
	{
		ChaosAbort( __FILE__, __LINE__ );
	}
	
	( *env )->CallStaticVoidMethod( env,adjust, onPause );
	DestroyJClass( adjust );
}
