#include "ChaosAndroid.h"
#include "NemesisJavaScript.h"
#include "AndroidPluginCommon.h"
#include "GrowthPush.h"

jclass GetGrowthPush()
{
	return CreateJClass( "siliconstudio/growthpush/GrowthPush" );
}

void GrowthPushInitialize()
{
	NemesisJavaScriptAppendCommand( NemesisJavaScriptPhaseUpdate, "GrowthPushInitializeForAndroid", GrowthPushInitializeForAndroid, 0 );
	NemesisJavaScriptAppendScript(
		"function GrowthPushInitializeForAndroid(applicationId,credentialId,senderId,releaseFlag,adInfoFlag)"
		"{Nemesis.PushCommand(Nemesis.UpdatePhase,\"GrowthPushInitializeForAndroid\");"
		"Nemesis.PushString(applicationId);Nemesis.PushString(credentialId);"
		"Nemesis.PushString(senderId);Nemesis.PushInteger(releaseFlag?1:0);Nemesis.PushInteger(adInfoFlag?1:0);};" );
	NemesisJavaScriptAppendScript( "function GrowthPushInitializeForIos(){};" );
	NemesisJavaScriptAppendCommand( NemesisJavaScriptPhaseUpdate, "GrowthPushTrackEvent", GrowthPushTrackEvent, 0 );
	NemesisJavaScriptAppendScript( "function GrowthPushTrackEvent(name){Nemesis.PushCommand(Nemesis.UpdatePhase,\"GrowthPushTrackEvent\");Nemesis.PushString(name);};" );
	NemesisJavaScriptAppendCommand( NemesisJavaScriptPhaseUpdate, "GrowthPushTrackEventValue", GrowthPushTrackEventValue, 0 );
	NemesisJavaScriptAppendScript( "function GrowthPushTrackEventValue(name,value){Nemesis.PushCommand(Nemesis.UpdatePhase,\"GrowthPushTrackEventValue\");Nemesis.PushString(name);Nemesis.PushString(value);};" );
	NemesisJavaScriptAppendCommand( NemesisJavaScriptPhaseUpdate, "GrowthPushSetTag", GrowthPushSetTag, 0 );
	NemesisJavaScriptAppendScript( "function GrowthPushSetTag(name){Nemesis.PushCommand(Nemesis.UpdatePhase,\"GrowthPushSetTag\");Nemesis.PushString(name);};" );
	NemesisJavaScriptAppendCommand( NemesisJavaScriptPhaseUpdate, "GrowthPushSetTagValue", GrowthPushSetTagValue, 0 );
	NemesisJavaScriptAppendScript( "function GrowthPushSetTagValue(name,value){Nemesis.PushCommand(Nemesis.UpdatePhase,\"GrowthPushSetTagValue\");Nemesis.PushString(name);Nemesis.PushString(value);};" );
}

void GrowthPushInitializeForAndroid( void* args )
{
	( void )args;

	JNIEnv* env = ChaosAndroidGetEnvironment();
	jclass growthPush = GetGrowthPush();

	char const* applicationId = NemesisJavaScriptPopString( NemesisJavaScriptPhaseUpdate );
	char const* credentialId = NemesisJavaScriptPopString( NemesisJavaScriptPhaseUpdate );
	char const* senderId = NemesisJavaScriptPopString( NemesisJavaScriptPhaseUpdate );
	int releaseFlag = NemesisJavaScriptPopInteger( NemesisJavaScriptPhaseUpdate );	
	int adInfoFlag = NemesisJavaScriptPopInteger( NemesisJavaScriptPhaseUpdate );
	
	jstring jApplicationId = ( *env )->NewStringUTF( env, applicationId );
	jstring jCredentialId = ( *env )->NewStringUTF( env, credentialId );
	jstring jSenderId = ( *env )->NewStringUTF( env, senderId );

	jmethodID initialize = ( *env )->GetStaticMethodID( env, growthPush, "Initialize", "(Landroid/app/Activity;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;II)V" );
	( *env )->CallStaticVoidMethod( env, growthPush, initialize, ThanatosAndroidGetActivityObject(), jApplicationId, jCredentialId, jSenderId, releaseFlag, adInfoFlag );
	
	( *env )->DeleteLocalRef( env, jSenderId );
	( *env )->DeleteLocalRef( env, jCredentialId );
	( *env )->DeleteLocalRef( env, jApplicationId );	
	DestroyJClass( growthPush );
}

void GrowthPushTrackEvent( void* args )
{
	( void )args;

	JNIEnv* env = ChaosAndroidGetEnvironment();
	jclass growthPush = GetGrowthPush();

	char const* name = NemesisJavaScriptPopString( NemesisJavaScriptPhaseUpdate );
	jstring jName = ( *env )->NewStringUTF( env, name );

	jmethodID trackEvent = ( *env )->GetStaticMethodID( env, growthPush, "TrackEvent", "(Ljava/lang/String;)V" );
	( *env )->CallStaticVoidMethod( env, growthPush, trackEvent, jName );
	
	( *env )->DeleteLocalRef( env, jName );	
	DestroyJClass( growthPush );
}

void GrowthPushTrackEventValue( void* args )
{
	( void )args;

	JNIEnv* env = ChaosAndroidGetEnvironment();
	jclass growthPush = GetGrowthPush();

	char const* name = NemesisJavaScriptPopString( NemesisJavaScriptPhaseUpdate );
	char const* value = NemesisJavaScriptPopString( NemesisJavaScriptPhaseUpdate );
	
	jstring jName = ( *env )->NewStringUTF( env, name );
	jstring jValue = ( *env )->NewStringUTF( env, value );

	jmethodID trackEventValue = ( *env )->GetStaticMethodID( env, growthPush, "TrackEventValue", "(Ljava/lang/String;Ljava/lang/String;)V" );
	( *env )->CallStaticVoidMethod( env, growthPush, trackEventValue, jName, jValue );
	
	( *env )->DeleteLocalRef( env, jValue );	
	( *env )->DeleteLocalRef( env, jName );	
	DestroyJClass( growthPush );
}

void GrowthPushSetTag( void* args )
{
	( void )args;

	JNIEnv* env = ChaosAndroidGetEnvironment();
	jclass growthPush = GetGrowthPush();

	char const* name = NemesisJavaScriptPopString( NemesisJavaScriptPhaseUpdate );
	jstring jName = ( *env )->NewStringUTF( env, name );

	jmethodID setTag = ( *env )->GetStaticMethodID( env, growthPush, "SetTag", "(Ljava/lang/String;)V" );
	( *env )->CallStaticVoidMethod( env, growthPush, setTag, jName );
	
	( *env )->DeleteLocalRef( env, jName );	
	DestroyJClass( growthPush );
}

void GrowthPushSetTagValue( void* args )
{
	( void )args;

	JNIEnv* env = ChaosAndroidGetEnvironment();
	jclass growthPush = GetGrowthPush();

	char const* name = NemesisJavaScriptPopString( NemesisJavaScriptPhaseUpdate );
	char const* value = NemesisJavaScriptPopString( NemesisJavaScriptPhaseUpdate );
	
	jstring jName = ( *env )->NewStringUTF( env, name );
	jstring jValue = ( *env )->NewStringUTF( env, value );

	jmethodID setTagValue = ( *env )->GetStaticMethodID( env, growthPush, "SetTagValue", "(Ljava/lang/String;Ljava/lang/String;)V" );
	( *env )->CallStaticVoidMethod( env, growthPush, setTagValue, jName, jValue );
	
	( *env )->DeleteLocalRef( env, jValue );	
	( *env )->DeleteLocalRef( env, jName );	
	DestroyJClass( growthPush );
}
